@@ -513,7 +513,7 @@ done_err:
     return FALSE;
 }
 
-static unsigned i386_map_dwarf_register(unsigned regno)
+static unsigned i386_map_dwarf_register(unsigned regno, BOOL eh_frame)
 {
     unsigned    reg;
 
@@ -523,8 +523,17 @@ static unsigned i386_map_dwarf_register(unsigned regno)
     case  1: reg = CV_REG_ECX; break;
     case  2: reg = CV_REG_EDX; break;
     case  3: reg = CV_REG_EBX; break;
-    case  4: reg = CV_REG_ESP; break;
-    case  5: reg = CV_REG_EBP; break;
+    case  4:
+    case  5:
+#ifdef __APPLE__
+        /* On OS X, DWARF eh_frame uses a different mapping for the registers.  It's
+           apparently the mapping as emitted by GCC, at least at some point in its history. */
+        if (eh_frame)
+            reg = (regno == 4) ? CV_REG_EBP : CV_REG_ESP;
+        else
+#endif
+            reg = (regno == 4) ? CV_REG_ESP : CV_REG_EBP;
+        break;
     case  6: reg = CV_REG_ESI; break;
     case  7: reg = CV_REG_EDI; break;
     case  8: reg = CV_REG_EIP; break;
