@@ -54,13 +54,29 @@ streambuf* __thiscall streambuf_setbuf(streambuf*, char*, int);
 void __thiscall streambuf_setg(streambuf*, char*, char*, char*);
 void __thiscall streambuf_setp(streambuf*, char*, char*);
 
+/* class ios */
+struct _ostream;
 typedef struct {
-    LPVOID VTable;
-} class_ios;
+    const vtable_ptr *vtable;
+    streambuf *sb;
+    ios_io_state state;
+    int special[4];
+    int delbuf;
+    struct _ostream *tie;
+    ios_flags flags;
+    int precision;
+    char fill;
+    int width;
+    int do_lock;
+    CRITICAL_SECTION lock;
+} ios;
 
-typedef struct {
-    LPVOID VTable;
-} class_ostream;
+ios* __thiscall ios_assign(ios*, const ios*);
+
+/* class ostream */
+typedef struct _ostream {
+    const vtable_ptr *vtable;
+} ostream;
 
 typedef struct {
     LPVOID VTable;
@@ -68,6 +84,8 @@ typedef struct {
 
 /* ??_7streambuf@@6B@ */
 extern const vtable_ptr MSVCP_streambuf_vtable;
+/* ??_7ios@@6B@ */
+extern const vtable_ptr MSVCP_ios_vtable;
 
 #ifndef __GNUC__
 void __asm_dummy_vtables(void) {
@@ -84,11 +102,14 @@ void __asm_dummy_vtables(void) {
             VTABLE_ADD_FUNC(streambuf_underflow)
             VTABLE_ADD_FUNC(streambuf_pbackfail)
             VTABLE_ADD_FUNC(streambuf_doallocate));
+    __ASM_VTABLE(ios,
+            VTABLE_ADD_FUNC(ios_vector_dtor));
 #ifndef __GNUC__
 }
 #endif
 
 DEFINE_RTTI_DATA0(streambuf, 0, ".?AVstreambuf@@")
+DEFINE_RTTI_DATA0(ios, 0, ".?AVios@@")
 
 /* ??0streambuf@@IAE@PADH@Z */
 /* ??0streambuf@@IEAA@PEADH@Z */
@@ -151,6 +172,8 @@ streambuf* __thiscall streambuf_assign(streambuf *this, const streambuf *rhs)
 
 /* ??_Estreambuf@@UAEPAXI@Z */
 DEFINE_THISCALL_WRAPPER(streambuf_vector_dtor, 8)
+#define call_streambuf_vector_dtor(this, flags) CALL_VTBL_FUNC(this, 0,\
+        streambuf*, (streambuf*, unsigned int), (this, flags))
 streambuf* __thiscall streambuf_vector_dtor(streambuf *this, unsigned int flags)
 {
     TRACE("(%p %x)\n", this, flags);
@@ -365,10 +388,13 @@ int __thiscall streambuf_overflow(streambuf *this, int c)
 /* ?pbackfail@streambuf@@UAEHH@Z */
 /* ?pbackfail@streambuf@@UEAAHH@Z */
 DEFINE_THISCALL_WRAPPER(streambuf_pbackfail, 8)
+#define call_streambuf_pbackfail(this, c) CALL_VTBL_FUNC(this, 36, int, (streambuf*, int), (this, c))
 int __thiscall streambuf_pbackfail(streambuf *this, int c)
 {
-    FIXME("(%p %d): stub\n", this, c);
-    return 0;
+    TRACE("(%p %d)\n", this, c);
+    if (this->gptr <= this->eback)
+        return EOF;
+    return *--this->gptr = c;
 }
 
 /* ?seekoff@streambuf@@UAEJJW4seek_dir@ios@@H@Z */
@@ -655,26 +681,504 @@ void __thiscall streambuf_stossc(streambuf *this)
     }
 }
 
-/******************************************************************
- *		 ??1ios@@UAE@XZ (MSVCRTI.@)
- *        class ios & __thiscall ios::-ios<<(void)
- */
-DEFINE_THISCALL_WRAPPER(MSVCIRT_ios_sl_void,4)
-void * __thiscall MSVCIRT_ios_sl_void(class_ios * _this)
+/* ?sputbackc@streambuf@@QAEHD@Z */
+/* ?sputbackc@streambuf@@QEAAHD@Z */
+DEFINE_THISCALL_WRAPPER(streambuf_sputbackc, 8)
+int __thiscall streambuf_sputbackc(streambuf *this, char ch)
 {
-   FIXME("(%p) stub\n", _this);
-   return _this;
+    TRACE("(%p %d)\n", this, ch);
+    return call_streambuf_pbackfail(this, ch);
+}
+
+/* ?dbp@streambuf@@QAEXXZ */
+/* ?dbp@streambuf@@QEAAXXZ */
+DEFINE_THISCALL_WRAPPER(streambuf_dbp, 4)
+void __thiscall streambuf_dbp(streambuf *this)
+{
+    printf("\nSTREAMBUF DEBUG INFO: this=%p, ", this);
+    if (this->unbuffered) {
+        printf("unbuffered\n");
+    } else {
+        printf("_fAlloc=%d\n", this->allocated);
+        printf(" base()=%p, ebuf()=%p,  blen()=%d\n", this->base, this->ebuf, streambuf_blen(this));
+        printf("pbase()=%p, pptr()=%p, epptr()=%d\n", this->pbase, this->pptr, this->epptr);
+        printf("eback()=%p, gptr()=%p, egptr()=%d\n", this->eback, this->gptr, this->egptr);
+    }
+}
+
+/* ??0ios@@IAE@ABV0@@Z */
+/* ??0ios@@IEAA@AEBV0@@Z */
+DEFINE_THISCALL_WRAPPER(ios_copy_ctor, 8)
+ios* __thiscall ios_copy_ctor(ios *this, const ios *copy)
+{
+    TRACE("(%p %p)\n", this, copy);
+    this->vtable = &MSVCP_ios_vtable;
+    this->sb = NULL;
+    this->delbuf = 0;
+    InitializeCriticalSection(&this->lock);
+    return ios_assign(this, copy);
+}
+
+/* ??0ios@@QAE@PAVstreambuf@@@Z */
+/* ??0ios@@QEAA@PEAVstreambuf@@@Z */
+DEFINE_THISCALL_WRAPPER(ios_sb_ctor, 8)
+ios* __thiscall ios_sb_ctor(ios *this, streambuf *sb)
+{
+    TRACE("(%p %p)\n", this, sb);
+    this->vtable = &MSVCP_ios_vtable;
+    this->sb = sb;
+    this->state = sb ? IOSTATE_goodbit : IOSTATE_badbit;
+    this->special[0] = this->special[1] = 0;
+    this->delbuf = 0;
+    this->tie = NULL;
+    this->flags = 0;
+    this->precision = 6;
+    this->fill = ' ';
+    this->width = 0;
+    this->do_lock = -1;
+    InitializeCriticalSection(&this->lock);
+    return this;
+}
+
+/* ??0ios@@IAE@XZ */
+/* ??0ios@@IEAA@XZ */
+DEFINE_THISCALL_WRAPPER(ios_ctor, 4)
+ios* __thiscall ios_ctor(ios *this)
+{
+    return ios_sb_ctor(this, NULL);
+}
+
+/* ??1ios@@UAE@XZ */
+/* ??1ios@@UEAA@XZ */
+DEFINE_THISCALL_WRAPPER(ios_dtor, 4)
+void __thiscall ios_dtor(ios *this)
+{
+    TRACE("(%p)\n", this);
+    if (this->delbuf && this->sb)
+        call_streambuf_vector_dtor(this->sb, 1);
+    this->sb = NULL;
+    this->state = IOSTATE_badbit;
+    DeleteCriticalSection(&this->lock);
+}
+
+/* ??4ios@@IAEAAV0@ABV0@@Z */
+/* ??4ios@@IEAAAEAV0@AEBV0@@Z */
+DEFINE_THISCALL_WRAPPER(ios_assign, 8)
+ios* __thiscall ios_assign(ios *this, const ios *rhs)
+{
+    TRACE("(%p %p)\n", this, rhs);
+    this->state = rhs->state;
+    if (!this->sb)
+        this->state |= IOSTATE_badbit;
+    this->tie = rhs->tie;
+    this->flags = rhs->flags;
+    this->precision = (char) rhs->precision;
+    this->fill = rhs->fill;
+    this->width = (char) rhs->width;
+    return this;
+}
+
+/* ??7ios@@QBEHXZ */
+/* ??7ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_op_not, 4)
+int __thiscall ios_op_not(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ??Bios@@QBEPAXXZ */
+/* ??Bios@@QEBAPEAXXZ */
+DEFINE_THISCALL_WRAPPER(ios_op_void, 4)
+void* __thiscall ios_op_void(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return NULL;
+}
+
+/* ??_Eios@@UAEPAXI@Z */
+DEFINE_THISCALL_WRAPPER(ios_vector_dtor, 8)
+ios* __thiscall ios_vector_dtor(ios *this, unsigned int flags)
+{
+    TRACE("(%p %x)\n", this, flags);
+    if (flags & 2) {
+        /* we have an array, with the number of elements stored before the first object */
+        INT_PTR i, *ptr = (INT_PTR *)this-1;
+
+        for (i = *ptr-1; i >= 0; i--)
+            ios_dtor(this+i);
+        MSVCRT_operator_delete(ptr);
+    } else {
+        ios_dtor(this);
+        if (flags & 1)
+            MSVCRT_operator_delete(this);
+    }
+    return this;
+}
+
+/* ??_Gios@@UAEPAXI@Z */
+DEFINE_THISCALL_WRAPPER(ios_scalar_dtor, 8)
+ios* __thiscall ios_scalar_dtor(ios *this, unsigned int flags)
+{
+    TRACE("(%p %x)\n", this, flags);
+    ios_dtor(this);
+    if (flags & 1) MSVCRT_operator_delete(this);
+    return this;
+}
+
+/* ?bad@ios@@QBEHXZ */
+/* ?bad@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_bad, 4)
+int __thiscall ios_bad(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?bitalloc@ios@@SAJXZ */
+LONG __cdecl ios_bitalloc(void)
+{
+    FIXME("() stub\n");
+    return 0;
+}
+
+/* ?clear@ios@@QAEXH@Z */
+/* ?clear@ios@@QEAAXH@Z */
+DEFINE_THISCALL_WRAPPER(ios_clear, 8)
+void __thiscall ios_clear(ios *this, int state)
+{
+    FIXME("(%p %d) stub\n", this, state);
+}
+
+/* ?clrlock@ios@@QAAXXZ */
+/* ?clrlock@ios@@QEAAXXZ */
+void __cdecl ios_clrlock(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+/* ?delbuf@ios@@QAEXH@Z */
+/* ?delbuf@ios@@QEAAXH@Z */
+DEFINE_THISCALL_WRAPPER(ios_delbuf_set, 8)
+void __thiscall ios_delbuf_set(ios *this, int delete)
+{
+    FIXME("(%p %d) stub\n", this, delete);
+}
+
+/* ?delbuf@ios@@QBEHXZ */
+/* ?delbuf@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_delbuf_get, 4)
+int __thiscall ios_delbuf_get(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?dec@@YAAAVios@@AAV1@@Z */
+/* ?dec@@YAAEAVios@@AEAV1@@Z */
+ios* __cdecl ios_dec(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return this;
+}
+
+/* ?eof@ios@@QBEHXZ */
+/* ?eof@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_eof, 4)
+int __thiscall ios_eof(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?fail@ios@@QBEHXZ */
+/* ?fail@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_fail, 4)
+int __thiscall ios_fail(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?fill@ios@@QAEDD@Z */
+/* ?fill@ios@@QEAADD@Z */
+DEFINE_THISCALL_WRAPPER(ios_fill_set, 8)
+char __thiscall ios_fill_set(ios *this, char fill)
+{
+    FIXME("(%p %d) stub\n", this, fill);
+    return EOF;
+}
+
+/* ?fill@ios@@QBEDXZ */
+/* ?fill@ios@@QEBADXZ */
+DEFINE_THISCALL_WRAPPER(ios_fill_get, 4)
+char __thiscall ios_fill_get(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return EOF;
+}
+
+/* ?flags@ios@@QAEJJ@Z */
+/* ?flags@ios@@QEAAJJ@Z */
+DEFINE_THISCALL_WRAPPER(ios_flags_set, 8)
+LONG __thiscall ios_flags_set(ios *this, LONG flags)
+{
+    FIXME("(%p %x) stub\n", this, flags);
+    return 0;
+}
+
+/* ?flags@ios@@QBEJXZ */
+/* ?flags@ios@@QEBAJXZ */
+DEFINE_THISCALL_WRAPPER(ios_flags_get, 4)
+LONG __thiscall ios_flags_get(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?good@ios@@QBEHXZ */
+/* ?good@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_good, 4)
+int __thiscall ios_good(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?hex@@YAAAVios@@AAV1@@Z */
+/* ?hex@@YAAEAVios@@AEAV1@@Z */
+ios* __cdecl ios_hex(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return this;
+}
+
+/* ?init@ios@@IAEXPAVstreambuf@@@Z */
+/* ?init@ios@@IEAAXPEAVstreambuf@@@Z */
+DEFINE_THISCALL_WRAPPER(ios_init, 8)
+void __thiscall ios_init(ios *this, streambuf *sb)
+{
+    TRACE("(%p %p)\n", this, sb);
+    if (this->delbuf && this->sb)
+        call_streambuf_vector_dtor(this->sb, 1);
+    this->sb = sb;
+    if (sb == NULL)
+        this->state |= IOSTATE_badbit;
+    else
+        this->state &= ~IOSTATE_badbit;
+}
+
+/* ?iword@ios@@QBEAAJH@Z */
+/* ?iword@ios@@QEBAAEAJH@Z */
+DEFINE_THISCALL_WRAPPER(ios_iword, 8)
+LONG* __thiscall ios_iword(const ios *this, int index)
+{
+    FIXME("(%p %d) stub\n", this, index);
+    return NULL;
+}
+
+/* ?lock@ios@@QAAXXZ */
+/* ?lock@ios@@QEAAXXZ */
+void __cdecl ios_lock(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+/* ?lockbuf@ios@@QAAXXZ */
+/* ?lockbuf@ios@@QEAAXXZ */
+void __cdecl ios_lockbuf(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+/* ?lockc@ios@@KAXXZ */
+void __cdecl ios_lockc(void)
+{
+    FIXME("() stub\n");
+}
+
+/* ?lockptr@ios@@IAEPAU_CRT_CRITICAL_SECTION@@XZ */
+/* ?lockptr@ios@@IEAAPEAU_CRT_CRITICAL_SECTION@@XZ */
+DEFINE_THISCALL_WRAPPER(ios_lockptr, 4)
+CRITICAL_SECTION* __thiscall ios_lockptr(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return NULL;
+}
+
+/* ?oct@@YAAAVios@@AAV1@@Z */
+/* ?oct@@YAAEAVios@@AEAV1@@Z */
+ios* __cdecl ios_oct(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return this;
+}
+
+/* ?precision@ios@@QAEHH@Z */
+/* ?precision@ios@@QEAAHH@Z */
+DEFINE_THISCALL_WRAPPER(ios_precision_set, 8)
+int __thiscall ios_precision_set(ios *this, int prec)
+{
+    FIXME("(%p %d) stub\n", this, prec);
+    return 0;
+}
+
+/* ?precision@ios@@QBEHXZ */
+/* ?precision@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_precision_get, 4)
+int __thiscall ios_precision_get(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?pword@ios@@QBEAAPAXH@Z */
+/* ?pword@ios@@QEBAAEAPEAXH@Z */
+DEFINE_THISCALL_WRAPPER(ios_pword, 8)
+void** __thiscall ios_pword(const ios *this, int index)
+{
+    FIXME("(%p %d) stub\n", this, index);
+    return NULL;
+}
+
+/* ?rdbuf@ios@@QBEPAVstreambuf@@XZ */
+/* ?rdbuf@ios@@QEBAPEAVstreambuf@@XZ */
+DEFINE_THISCALL_WRAPPER(ios_rdbuf, 4)
+streambuf* __thiscall ios_rdbuf(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return NULL;
+}
+
+/* ?rdstate@ios@@QBEHXZ */
+/* ?rdstate@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_rdstate, 4)
+int __thiscall ios_rdstate(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?setf@ios@@QAEJJ@Z */
+/* ?setf@ios@@QEAAJJ@Z */
+DEFINE_THISCALL_WRAPPER(ios_setf, 8)
+LONG __thiscall ios_setf(ios *this, LONG flags)
+{
+    FIXME("(%p %x) stub\n", this, flags);
+    return 0;
+}
+
+/* ?setf@ios@@QAEJJJ@Z */
+/* ?setf@ios@@QEAAJJJ@Z */
+DEFINE_THISCALL_WRAPPER(ios_setf_mask, 12)
+LONG __thiscall ios_setf_mask(ios *this, LONG flags, LONG mask)
+{
+    FIXME("(%p %x %x) stub\n", this, flags, mask);
+    return 0;
+}
+
+/* ?setlock@ios@@QAAXXZ */
+/* ?setlock@ios@@QEAAXXZ */
+void __cdecl ios_setlock(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+/* ?sync_with_stdio@ios@@SAXXZ */
+void __cdecl ios_sync_with_stdio(void)
+{
+    FIXME("() stub\n");
+}
+
+/* ?tie@ios@@QAEPAVostream@@PAV2@@Z */
+/* ?tie@ios@@QEAAPEAVostream@@PEAV2@@Z */
+DEFINE_THISCALL_WRAPPER(ios_tie_set, 8)
+ostream* __thiscall ios_tie_set(ios *this, ostream *ostr)
+{
+    FIXME("(%p %p) stub\n", this, ostr);
+    return NULL;
+}
+
+/* ?tie@ios@@QBEPAVostream@@XZ */
+/* ?tie@ios@@QEBAPEAVostream@@XZ */
+DEFINE_THISCALL_WRAPPER(ios_tie_get, 4)
+ostream* __thiscall ios_tie_get(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return NULL;
+}
+
+/* ?unlock@ios@@QAAXXZ */
+/* ?unlock@ios@@QEAAXXZ */
+void __cdecl ios_unlock(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+/* ?unlockbuf@ios@@QAAXXZ */
+/* ?unlockbuf@ios@@QEAAXXZ */
+void __cdecl ios_unlockbuf(ios *this)
+{
+    FIXME("(%p) stub\n", this);
+}
+
+/* ?unlockc@ios@@KAXXZ */
+void __cdecl ios_unlockc(void)
+{
+    FIXME("() stub\n");
+}
+
+/* ?unsetf@ios@@QAEJJ@Z */
+/* ?unsetf@ios@@QEAAJJ@Z */
+DEFINE_THISCALL_WRAPPER(ios_unsetf, 8)
+LONG __thiscall ios_unsetf(ios *this, LONG flags)
+{
+    FIXME("(%p %x) stub\n", this, flags);
+    return 0;
+}
+
+/* ?width@ios@@QAEHH@Z */
+/* ?width@ios@@QEAAHH@Z */
+DEFINE_THISCALL_WRAPPER(ios_width_set, 8)
+int __thiscall ios_width_set(ios *this, int width)
+{
+    FIXME("(%p %d) stub\n", this, width);
+    return 0;
+}
+
+/* ?width@ios@@QBEHXZ */
+/* ?width@ios@@QEBAHXZ */
+DEFINE_THISCALL_WRAPPER(ios_width_get, 4)
+int __thiscall ios_width_get(const ios *this)
+{
+    FIXME("(%p) stub\n", this);
+    return 0;
+}
+
+/* ?xalloc@ios@@SAHXZ */
+int __cdecl ios_xalloc(void)
+{
+    FIXME("() stub\n");
+    return 0;
 }
 
 /******************************************************************
  *		 ??0ostrstream@@QAE@XZ (MSVCRTI.@)
- *        class ostream & __thiscall ostrstream::ostrstream<<(void)
  */
-DEFINE_THISCALL_WRAPPER(MSVCIRT_ostrstream_sl_void,4)
-void * __thiscall MSVCIRT_ostrstream_sl_void(class_ostream * _this)
+DEFINE_THISCALL_WRAPPER(MSVCIRT_ostrstream_ctor,8)
+void * __thiscall MSVCIRT_ostrstream_ctor(ostream *this, BOOL virt_init)
 {
-   FIXME("(%p) stub\n", _this);
-   return _this;
+   FIXME("(%p %x) stub\n", this, virt_init);
+   return this;
+}
+
+/******************************************************************
+ *		 ??1ostrstream@@UAE@XZ (MSVCRTI.@)
+ */
+DEFINE_THISCALL_WRAPPER(MSVCIRT_ostrstream_dtor,4)
+void __thiscall MSVCIRT_ostrstream_dtor(ios *base)
+{
+    FIXME("(%p) stub\n", base);
 }
 
 /******************************************************************
@@ -682,7 +1186,7 @@ void * __thiscall MSVCIRT_ostrstream_sl_void(class_ostream * _this)
  *    class ostream & __thiscall ostream::operator<<(unsigned char)
  */
 DEFINE_THISCALL_WRAPPER(MSVCIRT_operator_sl_uchar,8)
-void * __thiscall MSVCIRT_operator_sl_uchar(class_ostream * _this, unsigned char ch)
+void * __thiscall MSVCIRT_operator_sl_uchar(ostream * _this, unsigned char ch)
 {
    FIXME("(%p)->(%c) stub\n", _this, ch);
    return _this;
@@ -693,7 +1197,7 @@ void * __thiscall MSVCIRT_operator_sl_uchar(class_ostream * _this, unsigned char
  *        class ostream & __thiscall ostream::operator<<(int)
  */
 DEFINE_THISCALL_WRAPPER(MSVCIRT_operator_sl_int,8)
-void * __thiscall MSVCIRT_operator_sl_int(class_ostream * _this, int integer)
+void * __thiscall MSVCIRT_operator_sl_int(ostream * _this, int integer)
 {
    FIXME("(%p)->(%d) stub\n", _this, integer);
    return _this;
@@ -704,7 +1208,7 @@ void * __thiscall MSVCIRT_operator_sl_int(class_ostream * _this, int integer)
  *    class ostream & __thiscall ostream::operator<<(char const *)
  */
 DEFINE_THISCALL_WRAPPER(MSVCIRT_operator_sl_pchar,8)
-void * __thiscall MSVCIRT_operator_sl_pchar(class_ostream * _this, const char * string)
+void * __thiscall MSVCIRT_operator_sl_pchar(ostream * _this, const char * string)
 {
    FIXME("(%p)->(%s) stub\n", _this, debugstr_a(string));
    return _this;
@@ -715,7 +1219,7 @@ void * __thiscall MSVCIRT_operator_sl_pchar(class_ostream * _this, const char *
  *    class ostream & __thiscall ostream::operator<<(class ostream & (__cdecl*)(class ostream &))
  */
 DEFINE_THISCALL_WRAPPER(MSVCIRT_operator_sl_callback,8)
-void * __thiscall MSVCIRT_operator_sl_callback(class_ostream * _this, class_ostream * (__cdecl*func)(class_ostream*))
+void * __thiscall MSVCIRT_operator_sl_callback(ostream * _this, ostream * (__cdecl*func)(ostream*))
 {
    TRACE("%p, %p\n", _this, func);
    return func(_this);
@@ -725,7 +1229,7 @@ void * __thiscall MSVCIRT_operator_sl_callback(class_ostream * _this, class_ostr
  *		?endl@@YAAAVostream@@AAV1@@Z (MSVCRTI.@)
  *           class ostream & __cdecl endl(class ostream &)
  */
-void * CDECL MSVCIRT_endl(class_ostream * _this)
+void * CDECL MSVCIRT_endl(ostream * _this)
 {
    FIXME("(%p)->() stub\n", _this);
    return _this;
@@ -735,7 +1239,7 @@ void * CDECL MSVCIRT_endl(class_ostream * _this)
  *		?ends@@YAAAVostream@@AAV1@@Z (MSVCRTI.@)
  *           class ostream & __cdecl ends(class ostream &)
  */
-void * CDECL MSVCIRT_ends(class_ostream * _this)
+void * CDECL MSVCIRT_ends(ostream * _this)
 {
    FIXME("(%p)->() stub\n", _this);
    return _this;
@@ -803,6 +1307,7 @@ static void init_io(void *base)
 {
 #ifdef __x86_64__
     init_streambuf_rtti(base);
+    init_ios_rtti(base);
 #endif
 }
 
