@@ -308,7 +308,7 @@ static unsigned char * handle_UserUnmarshal(ULONG *pFlags, unsigned char *pBuffe
     return pBuffer + sizeof(RemotableHandle);
 }
 
-static void handle_UserFree(ULONG *pFlags, HANDLE *phMenu)
+static void handle_UserFree(ULONG *pFlags, HANDLE *handle)
 {
     /* nothing to do */
 }
@@ -341,6 +341,9 @@ static void handle_UserFree(ULONG *pFlags, HANDLE *phMenu)
 IMPL_WIREM_HANDLE(HACCEL)
 IMPL_WIREM_HANDLE(HMENU)
 IMPL_WIREM_HANDLE(HWND)
+IMPL_WIREM_HANDLE(HDC)
+IMPL_WIREM_HANDLE(HICON)
+IMPL_WIREM_HANDLE(HBRUSH)
 
 /******************************************************************************
  *           HGLOBAL_UserSize [OLE32.@]
@@ -660,204 +663,6 @@ void __RPC_USER HBITMAP_UserFree(ULONG *pFlags, HBITMAP *phBmp)
     FIXME(":stub\n");
 }
 
-/******************************************************************************
- *           HICON_UserSize [OLE32.@]
- *
- * Calculates the buffer size required to marshal an icon.
- *
- * PARAMS
- *  pFlags       [I] Flags. See notes.
- *  StartingSize [I] Starting size of the buffer. This value is added on to
- *                   the buffer size required for the icon.
- *  phIcon       [I] Icon to size.
- *
- * RETURNS
- *  The buffer size required to marshal an icon plus the starting size.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to a ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of which
- *  the first parameter is a ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-ULONG __RPC_USER HICON_UserSize(ULONG *pFlags, ULONG StartingSize, HICON *phIcon)
-{
-    FIXME(":stub\n");
-    return StartingSize;
-}
-
-/******************************************************************************
-*           HICON_UserMarshal [OLE32.@]
-*
-* Marshals an icon into a buffer.
-*
-* PARAMS
-*  pFlags  [I] Flags. See notes.
-*  pBuffer [I] Buffer to marshal the icon into.
-*  phIcon  [I] Icon to marshal.
-*
-* RETURNS
-*  The end of the marshaled data in the buffer.
-*
-* NOTES
-*  Even though the function is documented to take a pointer to a ULONG in
-*  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of which
-*  the first parameter is a ULONG.
-*  This function is only intended to be called by the RPC runtime.
-*/
-unsigned char * __RPC_USER HICON_UserMarshal(ULONG *pFlags, unsigned char *pBuffer, HICON *phIcon)
-{
-    FIXME(":stub\n");
-    return pBuffer;
-}
-
-/******************************************************************************
- *           HICON_UserUnmarshal [OLE32.@]
- *
- * Unmarshals an icon from a buffer.
- *
- * PARAMS
- *  pFlags   [I] Flags. See notes.
- *  pBuffer  [I] Buffer to marshal the icon from.
- *  phIcon   [O] Address that receive the unmarshaled icon.
- *
- * RETURNS
- *  The end of the marshaled data in the buffer.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to an ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of which
- *  the first parameter is an ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-unsigned char * __RPC_USER HICON_UserUnmarshal(ULONG *pFlags, unsigned char *pBuffer, HICON *phIcon)
-{
-    FIXME(":stub\n");
-    return pBuffer;
-}
-
-/******************************************************************************
- *           HICON_UserFree [OLE32.@]
- *
- * Frees an unmarshaled icon.
- *
- * PARAMS
- *  pFlags   [I] Flags. See notes.
- *  phIcon   [I] Icon to free.
- *
- * RETURNS
- *  The end of the marshaled data in the buffer.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to a ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of
- *  which the first parameter is a ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-void __RPC_USER HICON_UserFree(ULONG *pFlags, HICON *phIcon)
-{
-    FIXME(":stub\n");
-}
-
-/******************************************************************************
- *           HDC_UserSize [OLE32.@]
- *
- * Calculates the buffer size required to marshal an HDC.
- *
- * PARAMS
- *  pFlags       [I] Flags. See notes.
- *  StartingSize [I] Starting size of the buffer. This value is added on to
- *                   the buffer size required for the clip format.
- *  phGlobal     [I] HDC to size.
- *
- * RETURNS
- *  The buffer size required to marshal an HDC plus the starting size.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to a ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of which
- *  the first parameter is a ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-ULONG __RPC_USER HDC_UserSize(ULONG *pFlags, ULONG StartingSize, HDC *phdc)
-{
-    FIXME(":stub\n");
-    return StartingSize;
-}
-
-/******************************************************************************
- *           HDC_UserMarshal [OLE32.@]
- *
- * Marshals an HDC into a buffer.
- *
- * PARAMS
- *  pFlags  [I] Flags. See notes.
- *  pBuffer [I] Buffer to marshal the clip format into.
- *  phdc    [I] HDC to marshal.
- *
- * RETURNS
- *  The end of the marshaled data in the buffer.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to a ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of which
- *  the first parameter is a ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-unsigned char * __RPC_USER HDC_UserMarshal(ULONG *pFlags, unsigned char *pBuffer, HDC *phdc)
-{
-    FIXME(":stub\n");
-    return pBuffer;
-}
-
-/******************************************************************************
- *           HDC_UserUnmarshal [OLE32.@]
- *
- * Unmarshals an HDC from a buffer.
- *
- * PARAMS
- *  pFlags   [I] Flags. See notes.
- *  pBuffer  [I] Buffer to marshal the clip format from.
- *  phdc     [O] Address that receive the unmarshaled HDC.
- *
- * RETURNS
- *  The end of the marshaled data in the buffer.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to an ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of which
- *  the first parameter is an ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-unsigned char * __RPC_USER HDC_UserUnmarshal(ULONG *pFlags, unsigned char *pBuffer, HDC *phdc)
-{
-    FIXME(":stub\n");
-    return pBuffer;
-}
-
-/******************************************************************************
- *           HDC_UserFree [OLE32.@]
- *
- * Frees an unmarshaled HDC.
- *
- * PARAMS
- *  pFlags   [I] Flags. See notes.
- *  phdc     [I] HDC to free.
- *
- * RETURNS
- *  The end of the marshaled data in the buffer.
- *
- * NOTES
- *  Even though the function is documented to take a pointer to a ULONG in
- *  pFlags, it actually takes a pointer to a USER_MARSHAL_CB structure, of
- *  which the first parameter is a ULONG.
- *  This function is only intended to be called by the RPC runtime.
- */
-void __RPC_USER HDC_UserFree(ULONG *pFlags, HDC *phdc)
-{
-    FIXME(":stub\n");
-}
-
 /******************************************************************************
  *           HPALETTE_UserSize [OLE32.@]
  *
