@@ -1560,6 +1560,11 @@ todo_wine {
     ok(my_alloc_called == 0, "alloc called %d\n", my_alloc_called);
 }
 
+    /* Prevent a memory leak when running with Wine.
+       Remove once the todo_wine block above is fixed. */
+    if (mem != mem_orig)
+        HeapFree(GetProcessHeap(), 0, mem_orig);
+
     my_free_called = 0;
     StubMsg.Buffer = StubMsg.BufferStart;
     NdrPointerFree( &StubMsg, mem, fmtstr_conf_str );
@@ -1590,7 +1595,6 @@ todo_wine {
        "mem not pointing at buffer %p/%p\n", mem, StubMsg.BufferStart + 12 );
     ok(my_alloc_called == 0, "alloc called %d\n", my_alloc_called);
 }
-    HeapFree(GetProcessHeap(), 0, mem_orig);
 
     my_alloc_called = 0;
     mem = mem_orig = HeapAlloc(GetProcessHeap(), 0, sizeof(memsrc));
